% This demo shows how kernel parallel analysis (kPA) can be used to
% estimate the gaussian scale and number of components to retain when doing
% gaussian kernel principal component analysis (KPCA). 
%
% A data set of two intermingled semi-circles in two dimensions is rotated
% to occupy 50 dimensions. Then kPA is used to estimate the number of
% components and gaussian scale, and those parameters are used to denoise 
% using KPCA.
%
% Reference: 
% K. W. Jrgensen and L. K. Hansen, 
% Model Selection for Gaussian Kernel PCA Denoising, 
% Neural Networks and Learning Systems, IEEE Transactions on, 
% vol. 23, no. 1, pp. 163-168, 2012.
%
% (C) Kasper Winther Jorgensen, IMM-DTU, 2012

clear
close all

%--------------------------------------------------------------------------
% create data set composed of 2 intermingled semi-circles
disp('loading data')
N            = 100;  % number of datapoints per semi-circle
noise_std    = 0.5;  % added noise standard deviation 
in.sigmavect = 2.5:0.5:6.5; % vector of investigated sigmas

radius    = 1;  % radius of semi-circles
[x,y,z]   = cylinder(radius,N*2);
moon1     = [x(1,1:N)-0.5; y(1,1:N)-0.25];
moon2     = [x(1,1:N)*-1+0.5; y(1,1:N)*-1+0.25];
X         = [moon1 moon2];
rot       = [[hamming(25); zeros(25,1)] [zeros(25,1); hamming(25)] zeros(50,48)];

in.X_train_o = [X; zeros(48,size(X,2))];
in.X_train_o = rot*in.X_train_o;
in.X_train   = in.X_train_o + randn(size(in.X_train_o))*noise_std;
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% run kernel parallel analysis to determine number of components and sigma
in.do_plot = 1;
[ncomp_kpa, sigma_kpa] = imm_kpca_kpa(in);
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% denoise using various pre-image algorithms

in.ncomp = ncomp_kpa;
in.sigma = sigma_kpa;

in.method = 'mika';
out_mika  = imm_kpca_preimage(in);

in.method   = 'honeine';
out_honeine = imm_kpca_preimage(in);

in.method = 'kwok';
out_kwok  = imm_kpca_preimage(in);

in.method = 'rathi';
out_rathi = imm_kpca_preimage(in);
%--------------------------------------------------------------------------


%--------------------------------------------------------------------------
% plotting
figure;
subplot(3,2,1)
imagesc(in.X_train_o)
title('Noisefree data')
ylabel('dim')
xlabel('N')

subplot(3,2,2)
imagesc(in.X_train)
title('Added noise')

subplot(3,2,3)
imagesc(out_mika.X_preimg);
title('denoised data using mika')

subplot(3,2,4)
imagesc(out_honeine.X_preimg);
title('denoised data using honeine')

subplot(3,2,5)
imagesc(out_kwok.X_preimg);
title('denoised data using kwok')

subplot(3,2,6)
imagesc(out_rathi.X_preimg);
title('denoised data using rathi')
%--------------------------------------------------------------------------
