function [ncomp_kpa, sigma_kpa] = imm_kpca_kpa(in)

% Kernel Parallel Analysis (kPA)
% USAGE:
%   [ncomp_kpa, sigma_kpa] = imm_kpca_kpa(in)
%
% INPUT:
%   in.X_train: input datapoints [dim X nsamples]
%   in.sigmavect: vector of scales to investigate. If not given, the best scale
%              will be estimated using a line-search
%
% OUTPUT:
%   ncomp_kpa: number of components to retain
%   sigma_kpa: scale value
%
%
% References:
%  K. W. Jrgensen and L. K. Hansen,
%  Model Selection for Gaussian Kernel PCA Denoising,
%  Neural Networks and Learning Systems, IEEE Transactions on,
%  vol. 23, no. 1, pp. 163-168, 2012.
%
% Kasper Winther Jrgensen, IMM, DTU, 2012


%--------------------------------------------------------------------------
% user specified parameters
p          = 49; % number of permutations
percentile = 95; % percentile for threshold
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% do not change below
num_ele         = numel(in.X_train);
[nvars, ncases] = size(in.X_train);
sigmavect       = in.sigmavect;
X_train         = in.X_train;

N   = size(X_train,2);
One = ones(N)/N;
H   = eye(N) - One;
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
disp('generating distance matrix')
d2 = imm_kpca_euclidean_dist(X_train',X_train');
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
disp('permuting data')
% create the permutation indices
% permute only within variables
perms    = zeros(p,nvars,ncases);
tmp_perm = reshape(1:num_ele, size(in.X_train));
for nds  = 1:p
    for n=1:nvars
        to_perm = tmp_perm(n,:);
        indx    = randperm(ncases);
        perms(nds,n,:) = to_perm(indx);
    end
    perm             = squeeze(perms(nds,:,:));
    X_perm           = in.X_train(perm);
    d2_perm(:,:,nds) = imm_kpca_euclidean_dist(X_perm',X_perm');
end
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
disp('search for optimal scale')

iter=1;
% iterate over sigmas
for c=sigmavect
    
    %----------------------------------------------------------------------
    % true data
    K   = exp(-0.5*d2/c^2);
    K_c = H*K*H;
    Lambda = sort(real(eig(K_c)),'descend');
    lambda(iter,:) = Lambda;
    %----------------------------------------------------------------------
    
    %----------------------------------------------------------------------
    % permuted data
    for i=1:p
        K_perm      = exp(-0.5*d2_perm(:,:,i)/c^2);
        K_c_perm    = H*K_perm*H;
        Lambda_perm = sort(real(eig(K_c_perm)),'descend');
        perm_lambda(i,:) = Lambda_perm;
    end
    %----------------------------------------------------------------------
    
    %----------------------------------------------------------------------
    % find the 95th percentile
    for i=1:size(perm_lambda,2)
        sorted = sort(perm_lambda(:,i));
        percentile_perm_lambda(iter,i) = sorted(round(percentile/100*length(sorted)));
    end
    
    diff = (lambda(iter,:)-percentile_perm_lambda(iter,:));
    indx = find(diff<0,1,'first')-1;
    if ~isempty(indx)
        npc(iter) = indx;
        nrg(iter) = sum(diff(1:npc(iter)));        
        
        fprintf(['scale=' num2str(c) ...
            ', energy diff=' num2str(nrg(iter)) ...
            ', #PCs=' num2str(npc(iter)) '\n'])
    end
    %--------------------------------------------------------------------------
    
    iter=iter+1;
end



%--------------------------------------------------------------------------
% get the best scale and number of components
[dummy, indx] = max(nrg);
ncomp_kpa     = npc(indx);
sigma_kpa     = sigmavect(indx);

fprintf(['Estimated number of components= ' num2str(ncomp_kpa) ', at sigma=' num2str(sigma_kpa) '\n' ]);

%--------------------------------------------------------------------------
% plotting
if in.do_plot
    figure
    subplot(2,1,1)
    plot(sigmavect,nrg,'-o','linewidth',2)
    ylabel('Energy - E','fontsize',18);
    xlabel('scale - \sigma','fontsize',18);
    grid on
    set(gca, 'linewidth',2)
    set(gca, 'fontsize',14)
    title('kernel parallel analysis')
    
    subplot(2,1,2)
    bar(sigmavect,npc,'linewidth',2)
    xlabel('scale - \sigma','fontsize',18);
    ylabel('# components - q','fontsize',18);
    grid on
    set(gca, 'linewidth',2)
    set(gca, 'fontsize',14)
    
    figure;
    hold on
    plot(lambda(indx,:),'linewidth',2);
    plot(percentile_perm_lambda(indx,:),'--r','linewidth',2);
    
    fill([1:ncomp_kpa ncomp_kpa:-1:1], [percentile_perm_lambda(indx,1:ncomp_kpa) ...
        lambda(indx,ncomp_kpa:-1:1) ],[0.8 0.8 0.8],'EdgeColor','none')
    
    % plot again to get on top
    plot(lambda(indx,:),'linewidth',2);
    plot(percentile_perm_lambda(indx,:),'--r','linewidth',2);
    
    box on
    xlim([0.5 ncomp_kpa+5])
    h=legend('\lambda','T');
    set(gca, 'linewidth',2)
    set(gca, 'fontsize',14)
    ylabel('Eigenvalue');
    xlabel('Component')
    title('kernel parallel analysis')
    set(h, 'fontsize',18)
    
end

