function p = imm_kpca_preimage(p)

% funcition p = imm_kpca_preimage(p)
%
% This script uses gaussian kernel principal component analysis to denoise
% data by solving the pre-image problem.
%
% USAGE:
%  p = imm_kpca_preimage(p)
%
% INPUT:
%  p.X_train  % training data [dim X nsamples]
%  p.X_test   % test data [dim X nsamples]
%  p.method   % pre-image algorithm, 'mika', 'kwok', 'honeine', 'rathi'
%  p.ncomp    % number of kernel principal components to use in preimage
%               estimation
%  p.sigma    % gaussian scale parameter 
%
% OUTPUT:
%  p.X_preimg     % Estimated pre-images (denoised data)
%  p.kernel_dist  % Distance from kernel point to projection of kernel point
%
%
% References:
%  K. W. Jrgensen and L. K. Hansen, 
%  Model Selection for Gaussian Kernel PCA Denoising, 
%  Neural Networks and Learning Systems, IEEE Transactions on, 
%  vol. 23, no. 1, pp. 163-168, 2012.
%
% pre-image algorithms:
%   mika:
%    S. Mika, B. Scholkopf, A. Smola, K.-R. Muller, M. Scholz, and G. Ratsch,
%    Kernel pca and de-noising in feature spaces, in Advances in Neural
%    Information Processing Systems 11. MIT Press, 1999, pp. 536542.
%
%   rathi:
%    Yogesh Rathi, Samuel Dambreville,  Allen Tannenbaum
%    'Statistical Shape Analysis using Kernel PCA', in SPIE, Electronic
%    Imaging, 2006
%
%   kwok:
%    J. T. Kwok and I. W. Tsang, The pre-image problem in kernel
%    methods, Neural Networks, IEEE Transactions on, vol. 15, no. 6,
%    pp. 15171525, November 2004
%
%   honeine:
%    Honeine, P.; Richard, C., "Solving the pre-image problem in kernel
%    machines: A direct method", Machine Learning for Signal Processing,
%    2009. MLSP 2009. IEEE International Workshop on.
%
%
% (C) Kasper Winther Jorgensen, IMM-DTU, 2012

%--------------------------------------------------------------------------
if ~isfield(p, 'method')
    p.method = 'mika';
    fprintf('method not specified, using ''mika''\n');
end

if ~isfield(p, 'random_start')
    p.random_start = 0;
    fprintf('random_start not specified, using ''0''\n');
end

% if X_test is not entered, use X_train
if ~isfield(p, 'X_test')
    p.X_test = p.X_train;    
end

if ~isfield(p, 'sigma')
    error('You must specify sigma, use e.g. imm_kpca_kpa.m to estimate')
end

if ~isfield(p, 'ncomp')
    error('You must specify ncomp, use e.g. imm_kpca_kpa.m to estimate')
end

%--------------------------------------------------------------------------
% parameters
X_train  = p.X_train;
X_test   = p.X_test;
method   = p.method;
ncomp    = p.ncomp;

[p.dim p.N] = size(X_train);

%--------------------------------------------------------------------------
% Compute kernel matrix K and center
p.c = p.sigma;
p.K = imm_kpca_kernel_matrix(X_train,X_train,p);
One = ones(p.N)/p.N;
p.H = eye(p.N) - One;
K_c = p.H*p.K*p.H; % Center K

%--------------------------------------------------------------------------
% Solve eigenvector equation
[a,l]  = eig(K_c);
Lambda = diag(real(l));

%--------------------------------------------------------------------------
% Sort the eigenvalues and the eigenvectors in descending order.
[Lambda indx] = sort(Lambda,1,'descend');
a        = a(:,indx);
p.Lambda = Lambda;
p.a = a;

%--------------------------------------------------------------------------
% Normalize eigenvectors
for k = 1:p.N
    if Lambda(k) ~= 0,
        a(:,k)=a(:,k)/sqrt(Lambda(k));
    end
end

%--------------------------------------------------------------------------
% Kernel projections
p.kernelproj = a(:,1:ncomp)' * K_c;

%--------------------------------------------------------------------------
% Estimate pre-images
p.M   = a(:,1:ncomp) * a(:,1:ncomp)';
p.K_c = K_c;

% create K_test
p.K_test = imm_kpca_kernel_matrix(X_train,X_test,p);
p.One    = ones(p.N,1)/p.N;

switch lower(method)
    case 'rathi'
        p = imm_kpca_preimg_rathi(p);
    case 'honeine'
        p = imm_kpca_preimg_honeine(p);
    case 'kwok'
        p = imm_kpca_preimg_kwok(p);
    case 'mika'
        p = imm_kpca_preimg_mika(p);    
end


